#ifndef _I386_MAGIC_H
#define _I386_MAGIC_H

#define I386_BLOCK_SIZE 4096	// 4KB
#define I386_POINTER_SIZE 4	// of analyzed system
#define I386_INTEGER_SIZE 4	// of analyzed system


// Border between kernelspace and userspace
// This value is OS dependent, and used only for 
// direct mapping, when PGD is not available
#define I386_KERNELSPACE_ADDR 0xC0000000 

// Maximal 32bit only address
#define I386_MAXADDR 0xFFFFFFFF

// Virtual Page Table (or PaGe Directory (PGD))
#define I386_VPT_MAGIC_ADDR 0x00000000	
// where to start looking for VPT 
// Used to be 0x00300000, but windows seems to store this table
// on lower addresses

// size of block with VPT
#define I386_VPT_MAGIC_BS 4096	

// how many page to check, when looking for VPT
#define I386_VPT_MAGIC_COUNT 500000	
// how many zeroes should be at the beginning of VPT (PGD)
#define I386_VPT_MAGIC_ZEROES 3072	
// how many non-zero values can be inside
#define I386_VPT_MAGIC_ZEROES_ERR 40	
// "zero" area. This is only heuristic, see documentation for more info.
//how many entries to check
#define I386_VPT_MAGIC_ENTRIES_CHECK	5	
// how many different values can be
#define I386_VPT_MAGIC_ENTRIES_ERR 1 //100	


// Bits   MBZ IGN A PCD PWT U/S R/W  P      (for 4KB PDE)
// Bits    1   D  A PCD PWT U/S R/W  P      (for 4MB PDE)
// 0x6b ~  0   1  1  0   1   0   1   1 
// 0x6f ~  0   1  1  0   1   1   1   1 
#define I386_VPT_MAGIC_MASK 0x6b
#define I386_VPT_MAGIC_USER_MASK 0x6f

// guess architecture magic

// This is PURE heuristics
// these values were correct for cca 50 various images I had
// IDT even does NOT have to be at 0x00000000

// Current approach check value of offset in target segment

// TODO add also chceck for MBZ bits ( 5, 6, 7, 12)
// This is strange, because IDT entries should have 64 bits
// I see them as 32 bits. (?)

// check only first 20 entries
#define I386_IDT_LEN 20
// check only certain bytes
#define I386_IDT_GUESS_MASK 0xFFFF0000
// most common value
#define I386_IDT_GUESS_MAGIC 0xF0000000
// how many errors to tolerate
#define	I386_IDT_GUESS_TRESHOLD I386_IDT_LEN/2

// TODO these constants are used only in alternative method of finding VPT
//#define I386_VPT_MAGIC_ERR 1		// how many error values can be found
//#define I386_VPT_MAGIC_ERR_FATAL 15 // stop immediately after FATAL errors
//#define I386_VPT_MIN_REC 1	// at least how many valid entries require
//#define I386_VPT_MAGIC_ERR_RATIO 3	// (#records/#errors) must be at least

// in used area of PGD (only some bits are checked)

#endif
