/*
 *             foriana
 * 
 *  FOrensic Ram Image ANAlyzer
 *  written in 2007-2010 by niekt0
 *  (niekt0@hysteria.sk, https://hysteria.sk/~niekt0)
 *  (please, mail me any discovered bugs/improvements.)
 * 
 * 
 *  version 1.0.0
 * 
 *  newest version at https://hysteria.sk/~niekt0/foriana
 * 
 *  Under GNU GPL.
 *  See http://gnu.org for more information.
 * 
 *  Use at your own risk.
 * 
*/


/* Limits of this version:
   see TODO file
 */

#include "globals.h"
#include "cache.h"
#include <arch/functions_allarch.h>
#include <os/functions_allos.h>

/*
 *  Global variables
 *  (little bit dirty)
 */


// Main structure
struct my_image img1;
// Architecture structures
struct arch_struct architectures[MAX_ARCHITECTURES];
// Operating systems structures
struct os_struct systems[MAX_OS];


char helpstatusfile[256]="foriana.cache"; //this will go to constants
char *statusfile2,*statusfile=helpstatusfile;
FILE *fs=NULL;		//cache file

char *inputfile;	//filename of analyzed image
char *process_mem_file=NULL;	//filename to dump process memory into
int inputfd,statusfd;	//input & status output 
FILE *f;		//separate descriptor for searching in file

struct stat sbuffer;    //for input file stats
char *errormsg;


int dbg=0; // global variable for dbgprint
int i,j,opt,mode=1,status=0;
u_int64_t read_addr=0,size_read_addr=0,pgd_read_addr=0;
u_int64_t single_process=0,single_process_phys=0,single_module=0;
unsigned char *pointer;

// for parsing cmdline options
int do_mod_list=0,do_mod_det=0,do_proc_det=0,do_proc_list=0;
int do_files_det=0,do_files_list=0,do_areas_list=1,ignore_cache=0,ddebug=0;
int do_create_proc_pattern=0,do_create_module_pattern=0;
int do_find_pgd=0;
int cache_readed=0;
//to save which foriana functionalities user requested

// Remember if user specified these values
int magic_module_set=0, magic_process_set=0, magic_user_process_set=0;


/******************************************************************************
 * THIS IS PLACE, 
 * where you should register your support for new ARCHITECTURE 
 *
 * register_all_arch()
 * return 0 on success
 *
 */
	
int register_all_arch(){

	// img1.arch=0 means architecture is unknown
	// If architecture is unknow, only few things can be done.
	img1.arch=0;
	strncpy(img1.architecture,"unknown",ARCH_NAME_LEN);
	img1.archcount=0;

	// Add new architecture HERE
	i386_init(&architectures[++img1.archcount]);
	x86_64_init(&architectures[++img1.archcount]);
	arm_init(&architectures[++img1.archcount]);

	// ADD HERE

	return 0;
}

/******************************************************************************
 * THIS IS PLACE, 
 * where you should register your support for new OPERATING SYSTEM
 *
 * register_all_os()
 * return 0 on success
 *
 */

int register_all_os(){

	// img1.os=0 means os is unknown
	// we can try to analyze system, even if OS is unknown
	// generic lists works well, and we can discover OS 
	// during search for structures.

	dbgprint("");
	img1.oscount=0;
	unknown_init(&systems[0]);

	// Add new operating system HERE
	linux_init(&systems[++img1.oscount]);
	windows_init(&systems[++img1.oscount]);
	bsd_init(&systems[++img1.oscount]);

	// ADD HERE

	// Initial values, we don't know nothing 
	// about system so far
	os_settings(&systems[0]);

	return 0;
}

/* 
 * Initialization function
 *
 * return > 0 if error occur
 * otherwise 0
 */

int init(){

	// Open input file	
        if ( (inputfd = open(inputfile,O_RDONLY)) == -1 ) {
                errormsg = strerror(errno);
                fprintf(stderr, "%s: %s.\n",inputfile,errormsg);
                return 21;
        }

	// Get file size
	if ( fstat(inputfd,&sbuffer) == -1){
		fprintf(stderr, "%s: Cannot get file statistics.\n",inputfile);
		return 22;
	}

	img1.ram_size=sbuffer.st_size;

	// Check if we can read image "header"
	if (img1.ram_size < MB_1){	
		fprintf(stderr, "%s: File is too small (<1MB). Are you sure "\
			"it is a memory dump?\n If yes, please modify foriana"\
			" source code.\n",inputfile);
		return 23;
	}
	
	printf("Dumped RAM size is %"PRIu64" MB.\n",img1.ram_size/MB_1);
	
	return 0;

}

void help(){
	int i;

	printf("\nforiana %s by niekt0@hysteria.sk\n",fversion); 
	printf("Usage: foriana [options] RAM_image\n");
	printf("Options:\n");
	printf("  -a/--all : try to dump everything available. Generally not good idea.\n");
	// Dynamic printing of supported architectures
	// (so they are registered on only one place)
	printf("  --arch <");
	for (i=1;i<=img1.archcount;i++) {
		dbgprint("archcount: %d, i: %d",img1.archcount,i);
		printf("%s",architectures[i].architecture);
		if (i != img1.archcount) { printf(",");}
	}
	printf("> : set architecture manually\n");
	printf("  -c/--cache <file> : specify cache file with already determined information\n");
	printf("  --create-process-pattern : try to create pattern that matches\n");
	printf("                             all process structures\n"); 
	printf("  --create-module-pattern : try to create pattern that matches \n");
	printf("                           all module structures\n");
	printf("  --debug: development debug informations will be printed\n");
	printf("  -d/--determine-only : only determine information about image,\n");
	printf("                        do not list anything\n");
	printf("  --determine-skip-processes : skip determination of process structure\n");
	printf("  --determine-skip-modules : skip determination of module structure\n");
	printf("  --determine-skip-files : skip determination of various file structures\n");
	printf("  --find-pgd : scan whole dump and list all pgd candidates. TODO\n");
	printf("  -h/--help : secret option\n");
	printf("  -i/--ignore-cache : ignores stored cache, and force new determination\n");
	printf("  --list-modules : force listing of modules\n");
	printf("  --list-processes : force listing of processes\n");
	printf("  --list-files : (TODO) force listing of open files\n");
	printf("  --list-skip-areas : do not list mapped areas for each process\n");
	printf("  --magic-module <name> : specify module name foriana is looking for\n");
	printf("                          (default: \"kthread\")\n");
	printf("  --magic-process <name> : specify process name foriana is looking for\n");
	printf("                           (default: \"yenta_socket\")\n");
	printf("  --magic-user-process <name> : specify process name foriana assumes first one\n");
	printf("                                (default: \"init\")\n");
	printf("  -m/--module <virtual address> : list information about this module only\n");
	printf("  -p/--process <virtual address> : list information about this process only\n");
	printf("  --process-physical <physical address> : same as -p, but for phys. address\n");
	printf("  --process-mem <file> : dump all mapped memory of this process into file\n");
	printf("  --pgd-read-linear <physical address of process VPT> : kernel PGD by default\n");
	printf("  --pgd <physical address of system VPT> : determined by default\n");
	printf("    specify address or use -1 (\"no\",\"hard\") for hardcoded translation\n");
//	printf("  --plugins <comma separated list of IDs of plugins to be run>:\n");
//	printf("             (options like -d are just aliases for plugin numbers \n");
	printf("  --read-linear <linear address> : use with --size-read-linear\n");
//	printf("  --show-plugins : list all available pluggins\n");
	// Dynamic printing of supported OS
	// (so they are registered at only one place)
	printf("  --system <");
	for (i=1;i<=img1.oscount;i++) {
		dbgprint("oscount: %d, i: %d",img1.oscount,i);
		printf("%s",systems[i].operating_system);
		if (i != img1.oscount) { printf(",");}
	}
	printf("> : Give foriana hint, what system is analyzed\n");
	printf("  --size-read-linear <size in bytes> : for --read-linear\n");
	printf("  -v/--version : print version number\n");
}

int main(int argc, char **argv)
{	

	// First things to do:

	// register architectures and systems, 
	// so we can write correct help messages, etc..
	register_all_arch();
	register_all_os();

	// get_parameters

	static struct option long_opts[]={
		{"all",0,0,'a'},
		{"arch",1,0,254},
		{"cache",1,0,'c'},
		{"create-process-pattern",0,0,261},
		{"create-module-pattern",0,0,262},
		{"debug",0,0,248},
		{"determine",0,0,'d'},    
		{"determine-skip-processes",0,0,245},
		{"determine-skip-modules",0,0,246},
		{"determine-skip-files",0,0,247},
		{"find-pgd",0,0,263},
		{"help",0,0,'h'},
		{"ignore-cache",0,0,'i'},
		{"list-files",0,0,249},
		{"list-modules",0,0,240},
		{"list-processes",0,0,241},	
		{"list-skip-areas",0,0,256},
		{"module",1,0,'m'},
		{"magic-module",1,0,242},
		{"magic-process",1,0,243},
		{"magic-user-process",1,0,244},
//		{"output",1,0,'o'}, // decided to use stdout instead 
		{"process",1,0,'p'},
		{"process-physical",1,0,275},
		{"process-mem",1,0,260},
		{"pgd-read-linear",1,0,250},
		{"pgd",1,0,270},
//		{"plugins",1,0,253},
		{"read-linear",1,0,'r'},
//		{"show-plugins",0,0,252},
		{"size-read-linear",1,0,'s'},
		{"system",1,0,251},
		{"version",0,0,'v'},
		{0,0,0,0}                             
	};

	
	/*
	 * Processing parameter in multiple runs is needed,
	 * because some parameters from cache can be overridden,
	 * so cache must be read before this parameter are processed.
	 * Unfortunately, cache filename is one of parameters, so we 
	 * need to extract filename 1 step earlier.
	 *
	 */


	// Terrible hack. I will die in hell for this.
	// 1.st iteration: If cache file should be ignored.
	while((opt = getopt_long(argc, argv, "ac:dhim:p:r:s:v",
				 long_opts, &i)) != -1) {
	 switch(opt) {
		case 'i': /*--ignore-cache*/
			ignore_cache=1; 
			break; /*--ignore-cache*/
	 }
	}

	// Get analyzed filename
        if ((argv[optind] != NULL) && (argv[optind+1]==NULL)) {
                printf("Analyzing file %s\n",argv[optind]);
		inputfile = argv[optind];
        }else {
		fprintf(stderr, "Syntax error.\n"); 
		help();
		return 7;	
	}

	optind=1;

	// 2.st iteration: Read cache file first, 
	// and then override user specified values 
	while((opt = getopt_long(argc, argv, "ac:dhim:p:r:s:v",
				 long_opts, &i)) != -1) {
	 switch(opt) {
		case 'c': /*--cache*/
			statusfile = optarg;
			break;
	 }
	}
	optind=1;

	/*
	 * Foriana check for cache file named "." + basename() of 
	 * analysed image inside working directory. 
         * If no cache file is found, new cache is
	 * created of this name. Name can be overridden with -c option.
	 *
	 */
	dbgprint("statusfile0: %s\n",statusfile);
	if ( statusfile == helpstatusfile ) {	// cache file not specified
		
		// have to copy string before calling basename,
		// according to man 3 basename
		if ( (statusfile2=strdup(inputfile) ) == NULL) {
			fprintf(stderr, "filename failed.\n");
			return 98;
		}	

		//max cache filename size
		snprintf(statusfile,256,".%s", basename(statusfile2)); 
		free(statusfile2);

	}



	// Read cache content.
	// This must be done before processing rest of arguments.
	
	dbgprint("Reading cache file");
	if ((ignore_cache > 0) || ((cache_readed = read_info()) != 0))
	{
		if (ignore_cache > 0) {
			printf("Ignoring cache file.\n");
		}else{
                	printf("N/A (%d).\n",cache_readed);
		}
	}


	while((opt = getopt_long(argc, argv, "ac:dhim:p:r:s:v",
				 long_opts, &i)) != -1) {
	  switch(opt) {
		case 'a': /*--all*/
			if (!(do_mod_det < 0)) {do_mod_det = 1; }
			if (!(do_mod_list < 0)) {do_mod_list = 1;}
			if (!(do_proc_det < 0)) {do_proc_det = 1;}
			if (!(do_proc_list < 0)) {do_proc_list = 1;}

			if (!(do_files_det < 0)) {do_files_det = 1;}
			if (!(do_files_list < 0)) {do_files_list = 1;}
			break;
		case 254: /*--arch*/
			
			// Check all the architectures, 
			// if one of names match.
			if (set_arch_from_list(optarg)!=0){ 
				printf("Unknown architecture: \"%s\".\n",
					optarg);
				return 4;
			}
			printf("User specified architecture: %s\n",
				img1.architecture);
			break;
		case 'c': /*--cache*/
			statusfile = optarg;
			break;
		case 261: /*--create-process-pattern*/
			do_proc_list=1; // listing of processes is required
			do_create_proc_pattern=1;
			break;
		case 262: /*--create-module-pattern*/
			do_mod_list=1; // listing of modules is required
			do_create_module_pattern=1;
			break;
		case 248: /*--debug*/
			ddebug=1;
			break;
		case 'd': /*--determine*/
			if (!(do_mod_det < 0)) {do_mod_det = 1;}
			if (!(do_proc_det < 0)) {do_proc_det = 1;}
			if (!(do_files_det < 0)) {do_files_det = 1;}
			break;
		case 'h': help(); return 0; break;
		case 'i': ignore_cache=1; break; /*--ignore-cache*/
		case 'm': /*--module*/
	 		if (mode == 1) {mode=MODE_MD; } else {mode=-MODE_MD;} 
			// --module is solo
			// hexa
			single_module=strtoull(optarg, NULL, MAX_PTR_LEN); 
			break; 
//		case 'o': outputfile=optarg; break; // Using stdout instead
//		case 253: /*--plugins*/ //TODO
//			printf("plugins: TODO\n");
//			return 15;
//			break;
		case 'p':/*--process*/
	 		if (mode == 1) {mode=MODE_PP; } else {mode=-MODE_PP;} 
			// --process is solo
			// hexa
			single_process=strtoull(optarg, NULL, MAX_PTR_LEN);
			break; 
		case 275:/*--process-physical*/
	 		if (mode == 1) {mode=MODE_PP; } else {mode=-MODE_PP;} 
			// --process is solo
			// hexa
			single_process_phys=strtoull(optarg,NULL,MAX_PTR_LEN);
			break; 

		case 260:/*--process-mem*/
			process_mem_file=optarg;
			break; 
		case 'r': /*--read-linear*/	
			if (mode == 1) {mode=MODE_RL; } else {mode=-MODE_RL;}
 			// hexa
			read_addr=strtoull(optarg, NULL, MAX_PTR_LEN);
			break;
//		case 252: /*--show-plugins"*/
//			printf("TODO");
//			break;
		case 's': /*--size-read-linear*/	
			size_read_addr=atol(optarg);
			break;
		case 251: /*--system*/	

			// Check all the OS, if one of names match.
			if (set_os_from_list(optarg)!=0){ 
				printf("Unknown OS, \"%s\".\n",optarg);
				return 0;
			}

			printf("User specified OS: %s\n",
				img1.operating_system);
			break;
		case 250: /*--pgd-read-linear*/
			// hexa
			pgd_read_addr=strtoull(optarg, NULL, MAX_PTR_LEN); 
			break;
		case 270: /*--pgd*/
			//hexa
			img1.VPT_address=strtoull(optarg, NULL, MAX_PTR_LEN); 
			img1.VPT_user_specified=1;
			if (strncmp("-1",optarg,2) == 0) {
				img1.VPT_not_available=1;
				img1.VPT_address=0;
			}
			if (strncmp("no",optarg,2) == 0) {
				img1.VPT_not_available=1;
				img1.VPT_address=0;
			}
			if (strncmp("hard",optarg,4) == 0) {
				img1.VPT_not_available=1;
				img1.VPT_address=0;
			}
			break;
		case 249: /*--list-files*/
			do_files_list=1; // TODO not working right now
			break;
		case 240: /*--list_modules*/
			do_mod_list=1;
			break;
		case 241: /*--list-processes*/
			do_proc_list=1;
			break;
		case 256: /*--list-skip-areas*/
			do_areas_list=0; 
			break;
		case 242: /*--magic-module*/
			magic_module_set=1;
			strncpy(img1.magic_module,optarg,MD_NAME_LEN);
			//to be sure
			img1.magic_module[MD_NAME_LEN -1]= '\0'; 
			printf("Using magic_module: %s\n",img1.magic_module);
			break;
		case 243: /*--magic-process*/
			magic_process_set=1;
			strncpy(img1.magic_process,optarg,TS_COMM_LEN);
 			//to be sure
			img1.magic_process[TS_COMM_LEN -1]= '\0';
			printf("Using magic_process: %s\n",
				img1.magic_process); 
			break;
		case 244: /*--magic-user-process*/
			magic_user_process_set=1;
			strncpy(img1.magic_user_process,optarg,TS_COMM_LEN);
			//to be sure
			img1.magic_user_process[TS_COMM_LEN -1]= '\0'; 
			printf("Using magic_user_process: %s\n",
				img1.magic_user_process);
			break;
		case 245: /*--determine-skip-processes*/ 
			do_proc_det=-1; // Conflict flag
			//	do_mod_list=-1; 
			// No listing without determination? 
			// It's less user-friendly, 
			// but allow you to modify cache file by hand.
			break;
		case 246: /*--determine-skip-modules*/;
			do_mod_det=-1; // Conflict flag
			//	do_mod_list=-1; 
			// No listing without determination? 
			// It's less user-friendly, 
			// but allow you to modify cache file by hand.
			break;
		case 247: /*determine-skip-files*/
			do_files_det=-1; // Conflict flag
			//	do_mod_list=-1; 
			// No listing without determination? 
			// It's less user-friendly, 
			// but allow you to modify cache file by hand.
			break;
		case 263: /*find-pgd*/
			do_find_pgd=1; 
			break;

		case 'v': /*--version*/
			printf("foriana %s\n",fversion); 
			return 0; 
			break; //;)
		case '?': // err
			fprintf(stderr, "Syntax error.\n"); 
			help(); 
			return 7; 
			break;	
	  }   	
	}	



	// parameter defaults

	// parameter collisions 

	/* Following parameter collisions are checked:
	 * - --ignore-cache, and --determine-skip-X and --list-X 
	 * - --read-linear and determine/list option
	 * - --process must be used solo
	 */
	
	if ((do_mod_det < 1) && (do_mod_list == 1) && (ignore_cache == 1)) {
		fprintf(stderr,
			"Parameter collision: Trying to list modules,\n "\
			"but determined information not available...exiting\n");
		return 13;
	}

	if ((do_proc_det < 1) && (do_proc_list == 1) && (ignore_cache == 1)) {
		fprintf(stderr,
			"Parameter collision: Trying to list processes,\n "\
			"but determined information not available...exiting\n");
		return 13;
	}

	if ((do_files_det < 1) && (do_files_list == 1) && (ignore_cache == 1)) {
		fprintf(stderr,
			"Parameter collision: Trying to list open files,\n "\
			"but determined information not available...exiting\n");
		return 13;
	}

	//read_linear
	if ((mode > 1) && ((do_mod_list == 1) || 
			(do_files_list == 1)  || 
			(do_proc_list == 1)   || 
			(do_mod_det == 1)     || 
			(do_files_det == 1)   || 
			(do_proc_det == 1)))  
	{		
		fprintf(stderr,"Parameter collision: some options "\
			"must be used solo...exiting\n");
		return 13;
	}
	
	//size_read_linear
	if ((mode == MODE_RL) && (size_read_addr == 0))  
	{	
		fprintf(stderr,"Parameter collision: size-read-linear "\
			"not specified...exiting\n");
		return 13;
	}


	// ----------------- program starts here ---------------------
	
	

	// Check size of input file & initialize
	if ((i = init()) != 0) {
		printf("Init returned %d. Exiting.\n",i);
		return 1;
	}


	if (ddebug == 1 ) {debug(); } //DBG
	fflush(stdout);

	// --find-pgd is solo
	if (do_find_pgd == 1) {
		// when using --find-pgd, --arch must be specified manually
		if (img1.arch == 0) {
			printf("Parameter collision: trying --find-pgd but "\
				"architecture not specified.. exiting\n");
			return 2;
		}

		// Special mode, only print (all) PGD candidates
		if ((i = img1.find_VPT(1)) != 0) { 
	                printf("find_VPT returned %d. Exiting.\n",i);
	                return 2;
	        }
		printf("Listed PGD candidates. Exiting.\n");
		return 0;
	}

	// Determine information only if ignore_cache is not set,
	// or if cache file is missing or invalid

	if ((ignore_cache > 0) || cache_readed !=0)
	{
			
		// Another (runtime) parameter collisions
		if ((do_mod_det < 1) && (do_mod_list == 1) )  {
			fprintf(stderr,
			 "Parameter collision: Trying to list modules,\n"\
			 "but determined information not available...exiting\n");

			return 14;
		}
		if ((do_proc_det < 1) && (do_proc_list == 1))  {
			fprintf(stderr,
			 "Parameter collision: Trying to list processes,\n"\
			 "but determined information not available...exiting\n");

			return 14;
		}
		if ((do_files_det < 1) && (do_files_list == 1))  {
			fprintf(stderr,
			 "Parameter collision: Trying to list open files,\n"\
			 "but determined information not available...exiting\n");

			return 14;
		}

		// If architecture was not specified
		if (img1.arch == 0) {
			if ( (i=guess_architecture()) ) {
				//something is wrong..
				printf("guess_architecture returned %d. "\
					"Exiting.\n",i);
				printf("Try to specify architecture manually"\
					 "using --arch option.\n");
				return 0;
			}
		}
		

		//Determine general information from image ()
		if ((i = img1.find_VPT(0)) != 0) { 
	                printf("find_VPT returned %d. Exiting.\n",i);
	                return 2;
	        }



		if ( do_proc_det > 0 ) {
			if ((i = proc_determine()) != 0) {
	        		printf("Linux_determine returned %d. "\
					"Exiting.\n",i);
       				return 3;
			}
        	}

		if ( do_mod_det > 0 ) {
			if ((i = lin_linux_determine_module()) != 0) {
        	       		printf("Linux_determine_module returned %d. "\
					"Exiting.\n",i);
                		return 4;
        		}
		}

		if ( do_files_det > 0 ) {
			printf("Determine open files: TODO\n");
//		        if ((i = i386_lin_determine_open_files(
//					img1.ts_one_of_lists)) != 0) {
//				printf("i386_lin_determine_open_files "
//					"returned %d. Exiting.\n",i);
//				return 6;
//			}
		}

		if ((i = save_info()) != 0) {
	                printf("save_info returned %d. Exiting.\n",i);
	                return 5;
	        }
		search_exit(); //for closing open file

	}
	fflush(stdout);
	
	/* -p/--process <hex_addr>
	 * Display: 
	 * - process linear address
	 * - process physical address
	 * - process name
	 * - process PGD (for --read-linear)
	 * - list of open files for this process
	 */
	if (mode == MODE_PP) { // --process is solo
		// Put this into function (for multiple architectures))
		if ((single_process == 0) && (single_process_phys == 0)) {
			printf("Invalid process address.\n");
			return 902;
		}
	
		return print_process_info(single_process,
					single_process_phys,
					process_mem_file);
	}

	/* -m/--module <hex_addr>
	 * Display: 
	 * - module address
	 * - module physical address
	 * - module name
	 * - ?
	 */
	if (mode == MODE_MD) { // --module is solo
		if (single_module == 0) {
			printf("Invalid module address.\n");
			return 802;
		}

		return print_module_info(single_module);

	}

	/* --read-linear <hex_addr>
	 * Display: 
	 * - linear address
	 * - physical address (first block)
	 * - hex dump
	 * - ascii dump (printable chars)
	 */
	if (mode == MODE_RL) { // read_linear is solo

		u_int64_t phys_addr_MODE_RL=0;
		
		printf("Reading %"PRIu64" bytes from linear address 0x%"PRIx64".\n",
			size_read_addr, read_addr);
		if ((pointer=(unsigned char *)malloc(size_read_addr))==NULL){ 
			// Not ideal solution for big blocks...
			fprintf(stderr, "malloc() failed.\n");
			return 724;
		}

		// if not specified, pgd_read_addr is 0 (kernel pgd)
		if ( (i = img1.read_linear(pgd_read_addr, 
						read_addr, 
						pointer, 
						size_read_addr,
						&phys_addr_MODE_RL) == -1 )) {
       	        	printf("--read-linear: 0x%"PRIx64" seems is not valid "\
				"virtual address.\n", read_addr);
			free(pointer);
    	       		return(723);
	       	} else {
			if (phys_addr_MODE_RL == 0) { //not mapped
				printf("0x%"PRIx64" does not seem to be mapped "\
					"to physical memory.\n", read_addr);
				free(pointer);
				return(742);
			} else {
				printf("Read %"PRIu64" bytes from 0x%"PRIx64" (physical"\
					" address 0x%"PRIx64"):\n", size_read_addr,
					read_addr, phys_addr_MODE_RL);
			}
		}
		
		// now print block we have read
		printf("Hex dump & printable strings:\n");

		// print is in function
		hexprint(pointer, size_read_addr, read_addr);

		printf("End of hexdump.\n");	
		free(pointer);
		return 0;
	}

	//Listing stuff
	
	if ( do_proc_list > 0 ) {
		if ((i = list_processes(img1.ts_one_of_lists,
					img1.ts_hl_comm_dst)) != 0) {
        	        printf("Listed %d processes (hopefully).\n",i);
		}
        }

 	if ( do_mod_list > 0 ) {
		if ((i = lin_list_modules(img1.md_one_of_lists,
					img1.md_hl_name_dst)) != 0) {
        	        printf("Listed %d modules (hopefully).\n",i);
        	}
	}

 	if ( do_files_list > 0 ) {
		printf("List open files: TODO (not working)\n");
//		if ((i = lin_list_files(img1.md_one_of_lists,
//				img1.md_hl_name_dst)) != 0) {
//        	        printf("Listed %d open files (hopefully).\n",i);
//        	}
	}

	printf("Clean exit;).\n");
	return 0;

}
