#ifndef _ALLOS_H
#define _ALLOS_H

// Because code for analyze of various OS is heavily redundant
// All constants are stored in one file

// Increase, if need support for more operating systems;)
#define MAX_OS 10

#define LINUX_MAGIC_PROCESS "kthread" 
// process, that should be found in all 2.6 running linuxes.
// "kthreadd" on some systems, but this is ok
// EDIT: probably not on arm ;(

#define LINUX_MAGIC_USER_PROCESS "init"
// user process, that should be found in all 2.6 running linuxes.
// We need this, when looking for task_struct that have valid userspace PGD

#define LINUX_MAGIC_MODULE "yenta_socket"
// module, that should be found in most of 2.6 running linuxes.
// (if there are wnough modules) 
// I have not found module present on ALL systems,
// probably you will need to specify this by parameter

#define WINDOWS_MAGIC_PROCESS "svchost.exe" 
// process, that should be found in all running windows systems.

#define WINDOWS_MAGIC_USER_PROCESS ""
// not needed

#define WINDOWS_MAGIC_MODULE ""
// not implemented

#define BSD_MAGIC_PROCESS "init" 
// process, that should be found in all "unix" systems.

#define BSD_MAGIC_USER_PROCESS ""
// not needed

#define BSD_MAGIC_MODULE ""
// not implemented

#define UNKNOWN_MAGIC_PROCESS LINUX_MAGIC_PROCESS
#define UNKNOWN_MAGIC_USER_PROCESS LINUX_MAGIC_USER_PROCESS
#define UNKNOWN_MAGIC_MODULE LINUX_MAGIC_MODULE


// The logic behind this is: 
// if not specified, dump is considered to be linux, 
// if looking for linux process fail, 
// dump is considered to be windows image 
// and search starts again



// MAGIC section...
// Linux specials

// ----- task_struct -----

// distance in list_head occurrences between name string
// and linked list in process structure
#define LINUX_TS_MAGIC_HEADLIST_NUMBER 3 // 9 //10

// How far from TS_MAGIC_HEADLIST_FRONT_NUMBER to look 
// for right one list_head 
// (both sides, unit: list_heads;)
// See theoretical explanation for more information.
#define LINUX_TS_MAGIC_HEADLIST_SPREAD 16 // 6
#define LINUX_TS_MAGIC_HEADLIST_FRONT_NUMBER 5 //3 

// At least how many processes must be in process list
#define LINUX_TS_MAGIC_MIN_LIST_SIZE 10

// How many times try to find list_head inside process structure
#define LINUX_TS_MAGIC_MAX_TRIES 350 //200 //160		

// distance from "tasks" list_head to "mm" mmstruct
#define LINUX_TS_MAGIC_HL_MM_DST_P 8

// Maximally how many errors can be in head_list
#define LINUX_TS_MAGIC_LIST_MAX_ERR 15

// Max. how many list_head we expect before name string 
// in process structure
#define LINUX_TS_MAGIC_HEADLIST_MAX 20


// ----- mm_struct -----
// distance from beginning of mm_struct to pgd variable...
#define LINUX_MM_MAGIC_PGD_DST_P 18

// ----- module -----
// distance in list_head occurrences between 
// list_head and name in module struct :) 
// actually not used, because they are just near
#define LINUX_MD_MAGIC_HEADLIST_NUMBER 1
#define LINUX_MD_MAGIC_MAX_TRIES 1
#define LINUX_MD_MAGIC_HL_NAME_DST_P 6
#define LINUX_MD_MAGIC_HEADLIST_MAX 10
#define LINUX_MD_PATTERN_SPREAD 300

// Maximal size of analysed list
#define LINUX_MAX_LIST_SIZE	65535

// ---------- BSD -----------
#define BSD_TS_MAGIC_HEADLIST_NUMBER	LINUX_TS_MAGIC_HEADLIST_NUMBER
#define BSD_TS_MAGIC_HEADLIST_SPREAD	LINUX_TS_MAGIC_HEADLIST_SPREAD
#define BSD_TS_MAGIC_HEADLIST_FRONT_NUMBER	LINUX_TS_MAGIC_HEADLIST_FRONT_NUMBER
#define BSD_TS_MAGIC_MIN_LIST_SIZE	LINUX_TS_MAGIC_MIN_LIST_SIZE
#define BSD_TS_MAGIC_MAX_TRIES		LINUX_TS_MAGIC_MAX_TRIES
#define BSD_TS_MAGIC_HL_MM_DST_P	LINUX_TS_MAGIC_HL_MM_DST_P
#define BSD_TS_MAGIC_LIST_MAX_ERR	LINUX_TS_MAGIC_LIST_MAX_ERR
#define BSD_TS_MAGIC_HEADLIST_MAX	LINUX_TS_MAGIC_HEADLIST_MAX
#define BSD_MM_MAGIC_PGD_DST_P		LINUX_MM_MAGIC_PGD_DST_P
#define BSD_MD_MAGIC_HEADLIST_NUMBER	LINUX_MD_MAGIC_HEADLIST_NUMBER
#define BSD_MD_MAGIC_MAX_TRIES		LINUX_MD_MAGIC_MAX_TRIES
#define BSD_MD_MAGIC_HL_NAME_DST_P	LINUX_MD_MAGIC_HL_NAME_DST_P
#define BSD_MD_MAGIC_HEADLIST_MAX	LINUX_MD_MAGIC_HEADLIST_MAX
#define BSD_MD_PATTERN_SPREAD		LINUX_MD_PATTERN_SPREAD
#define BSD_MAX_LIST_SIZE		LINUX_MAX_LIST_SIZE

// ---------- WINDOWS -----------
#define WINDOWS_TS_MAGIC_HEADLIST_NUMBER	LINUX_TS_MAGIC_HEADLIST_NUMBER
#define WINDOWS_TS_MAGIC_HEADLIST_SPREAD	LINUX_TS_MAGIC_HEADLIST_SPREAD
#define WINDOWS_TS_MAGIC_HEADLIST_FRONT_NUMBER	LINUX_TS_MAGIC_HEADLIST_FRONT_NUMBER
#define WINDOWS_TS_MAGIC_MIN_LIST_SIZE		LINUX_TS_MAGIC_MIN_LIST_SIZE
#define WINDOWS_TS_MAGIC_MAX_TRIES		LINUX_TS_MAGIC_MAX_TRIES
#define WINDOWS_TS_MAGIC_HL_MM_DST_P		LINUX_TS_MAGIC_HL_MM_DST_P
#define WINDOWS_TS_MAGIC_LIST_MAX_ERR		LINUX_TS_MAGIC_LIST_MAX_ERR
#define WINDOWS_TS_MAGIC_HEADLIST_MAX		LINUX_TS_MAGIC_HEADLIST_MAX
#define WINDOWS_MM_MAGIC_PGD_DST_P		LINUX_MM_MAGIC_PGD_DST_P
#define WINDOWS_MD_MAGIC_HEADLIST_NUMBER	LINUX_MD_MAGIC_HEADLIST_NUMBER
#define WINDOWS_MD_MAGIC_MAX_TRIES		LINUX_MD_MAGIC_MAX_TRIES
#define WINDOWS_MD_MAGIC_HL_NAME_DST_P		LINUX_MD_MAGIC_HL_NAME_DST_P
#define WINDOWS_MD_MAGIC_HEADLIST_MAX		LINUX_MD_MAGIC_HEADLIST_MAX
#define WINDOWS_MD_PATTERN_SPREAD		LINUX_MD_PATTERN_SPREAD
#define WINDOWS_MAX_LIST_SIZE			LINUX_MAX_LIST_SIZE

// ---------- UNKNOWN -----------
#define UNKNOWN_TS_MAGIC_HEADLIST_NUMBER	LINUX_TS_MAGIC_HEADLIST_NUMBER
#define UNKNOWN_TS_MAGIC_HEADLIST_SPREAD	LINUX_TS_MAGIC_HEADLIST_SPREAD
#define UNKNOWN_TS_MAGIC_HEADLIST_FRONT_NUMBER	LINUX_TS_MAGIC_HEADLIST_FRONT_NUMBER
#define UNKNOWN_TS_MAGIC_MIN_LIST_SIZE		LINUX_TS_MAGIC_MIN_LIST_SIZE
#define UNKNOWN_TS_MAGIC_MAX_TRIES		LINUX_TS_MAGIC_MAX_TRIES
#define UNKNOWN_TS_MAGIC_HL_MM_DST_P		LINUX_TS_MAGIC_HL_MM_DST_P
#define UNKNOWN_TS_MAGIC_LIST_MAX_ERR		LINUX_TS_MAGIC_LIST_MAX_ERR
#define UNKNOWN_TS_MAGIC_HEADLIST_MAX		LINUX_TS_MAGIC_HEADLIST_MAX
#define UNKNOWN_MM_MAGIC_PGD_DST_P		LINUX_MM_MAGIC_PGD_DST_P
#define UNKNOWN_MD_MAGIC_HEADLIST_NUMBER	LINUX_MD_MAGIC_HEADLIST_NUMBER
#define UNKNOWN_MD_MAGIC_MAX_TRIES		LINUX_MD_MAGIC_MAX_TRIES
#define UNKNOWN_MD_MAGIC_HL_NAME_DST_P		LINUX_MD_MAGIC_HL_NAME_DST_P
#define UNKNOWN_MD_MAGIC_HEADLIST_MAX		LINUX_MD_MAGIC_HEADLIST_MAX
#define UNKNOWN_MD_PATTERN_SPREAD		LINUX_MD_PATTERN_SPREAD
#define UNKNOWN_MAX_LIST_SIZE			LINUX_MAX_LIST_SIZE


#endif
