/*
 *	Functions for analysing bsd systems
*/

#include "../../globals.h"
#include "../functions_allos.h"
#include "../allos.h"
#include "magic.h"
#include "../../arch/functions_allarch.h"



/*
 * Hook for proc_determine
 *
 * bsd_hook_proc_determine (pointer to process address)
 * return 0 on success 
 */

int bsd_hook_proc_determine(u_int64_t *process){
	
	return 0;
}

/*
 * Hook to determine aditional information,
 * once process structure is found.
 *
 * bsd_hook_proc_determine2 ()
 * return 0 on success 
 */

int bsd_hook_proc_determine2(){
	
	return 0;
}

/*
 * Hook for listing processes
 *
 * For now, control only VMA listing
 * Another hook may be needed for future.
 *
 * bsd_hook_list_processes (pointer to VMA switch)
 * return 0 on success 
 */

int bsd_hook_list_processes(int *list_areas){

	if (do_areas_list) {
		printf("Disabling VMA listing, "\
			"not implemented for BSD kernels.\n");
		*list_areas=0;
	}

	return 0;
}

/*
 * Hook for matching process structure
 *
 * bsd_hook_verify_task_struct ( 
	pointer to hl size, 
	index in hl list,
	ver_addr (needed for bsd list,
	pointer to bsd list start
	pointer to found_os)
 * return 0 on success 
 */

int bsd_hook_verify_task_struct(int *hl_size, 
				int i, 
				u_int64_t ver_addr, 
				u_int64_t *bsd_list_start,
				int *found_os, 
				int *max_bsd_list_strings_size) 
{

	int bsd_hl_size=0; 
	int bsd_list_strings_size=0;
	u_int64_t bsd_list_start_tmp=0;

	// Non-Cyclic (BSD) style list_head 
	bsd_hl_size=bsd_list_head_size(img1.ts_headlists[i],
			&bsd_list_start_tmp,
			ver_addr - img1.ts_headlists[i] + img1.pointer_size,
			&bsd_list_strings_size);

	dbgprint("BSD list size: %d, strings size: %d", bsd_hl_size,
						bsd_list_strings_size);
	if (bsd_hl_size >= *hl_size) {
		// more and not loop (loop < 0)
		/* Sometimes, there are multiple pointers to list_head
		   we are looking for.
		   Therefore our used offset can be wrong, 
		   and we should count string for each element of list.
		 */

		dbgprint("Saving local maximum:(1.try) 0x%"PRIX64"",
				img1.ts_headlists[i]);

		if (	(bsd_hl_size > *hl_size) || 
			(bsd_list_strings_size > *max_bsd_list_strings_size)) { 

			dbgprint("Saving local maximum: 0x%"PRIX64", "\
					"bsd_hl: %d", img1.ts_headlists[i], 
					bsd_hl_size);
			dbgprint("previous size: %d, strings: %d, "\
					"prev strings: %d", *hl_size, 
					bsd_list_strings_size, 
					*max_bsd_list_strings_size);


			//save which list_head is right one
			img1.ts_headlists_number=i; 
	
			// remember local maximum
			*hl_size=bsd_hl_size;		
			//remember first entry of list
			*bsd_list_start=bsd_list_start_tmp; 
			*max_bsd_list_strings_size=bsd_list_strings_size;
			
		}
	}

	return 0;
}



/*
 * Hook for matching process structure
 *
 * bsd_hook_verify_task_struct2 ( 
 *	found_os, start of non-cyclic hl, pointer to hl )
 *
 * return 0 on success 
 */

int bsd_hook_verify_task_struct2(int found_os, u_int64_t bsd_list_start, u_int64_t *list_start) {

	// special case, bsd list is not circular
	// so we need to point to the first entry
	if (bsd_list_start != 0) {
		*list_start = bsd_list_start;
	}
	return 0;

}

/*
 * Function that sets pointers to all other 
 * os dependent functions and hooks
 *
 * bsd_init (pointer to os structure)
 * return 0 on success 
 */

int bsd_init(struct os_struct *self){

	strncpy(self->operating_system,"bsd\0",OS_NAME_LEN);
	strncpy(self->magic_process_name,BSD_MAGIC_PROCESS,TS_COMM_LEN);
	strncpy(self->magic_user_process_name,BSD_MAGIC_USER_PROCESS,TS_COMM_LEN);
	strncpy(self->magic_module_name,BSD_MAGIC_MODULE,MD_NAME_LEN);

	self->hook_proc_determine=(int (*)())bsd_hook_proc_determine;
	self->hook_proc_determine2=(int (*)())bsd_hook_proc_determine2;
	self->hook_list_processes=(int (*)())bsd_hook_list_processes;
	self->hook_verify_task_struct=(int (*)())bsd_hook_verify_task_struct;
	self->hook_verify_task_struct2=(int (*)())bsd_hook_verify_task_struct2;

	
	self->ts_magic_headlist_number = BSD_TS_MAGIC_HEADLIST_NUMBER;
	self->ts_magic_headlist_spread = BSD_TS_MAGIC_HEADLIST_SPREAD;
	self->ts_magic_headlist_front_number = BSD_TS_MAGIC_HEADLIST_FRONT_NUMBER;
	self->ts_magic_min_list_size = BSD_TS_MAGIC_MIN_LIST_SIZE;
	self->ts_magic_max_tries = BSD_TS_MAGIC_MAX_TRIES;
	self->ts_magic_hl_mm_dst_p = BSD_TS_MAGIC_HL_MM_DST_P;
	self->ts_magic_list_max_err = BSD_TS_MAGIC_LIST_MAX_ERR;
	self->ts_magic_headlist_max = BSD_TS_MAGIC_HEADLIST_MAX;
	self->mm_magic_pgd_dst_p = BSD_MM_MAGIC_PGD_DST_P;
	self->md_magic_headlist_number = BSD_MD_MAGIC_HEADLIST_NUMBER;
	self->md_magic_max_tries = BSD_MD_MAGIC_MAX_TRIES;
	self->md_magic_hl_name_dst_p = BSD_MD_MAGIC_HL_NAME_DST_P;
	self->md_magic_headlist_max = BSD_MD_MAGIC_HEADLIST_MAX;
	self->md_pattern_spread = BSD_MD_PATTERN_SPREAD;
	self->max_list_size = BSD_MAX_LIST_SIZE;

	return 0;
}
