
/*
 *	Functions for generic image analysing 
 */

#include "../globals.h"
#include "linux/functions_linux.h"
#include "windows/functions_windows.h"
#include "bsd/functions_bsd.h"
#include "unknown/functions_unknown.h"

/*
 * Function that sets operating system specific 
 * settings according to values in supplied structure
 *
 * os_settings (pointer to operating system struct)
 * return 0 on success, otherwise error > 0
 * results are stored in global structure
 *
 */

int os_settings(struct os_struct *os_to_set){

	strncpy(img1.operating_system,
		os_to_set->operating_system,
		OS_NAME_LEN);

	dbgprint("OS: %s",img1.operating_system);
	dbgprint("magic_process_set: %d",magic_process_set);

	// Set "magic" process name, only if user did not specified one.
	if ( magic_process_set == 0) {
		strncpy(img1.magic_process,
			os_to_set->magic_process_name,
			TS_COMM_LEN);
	}

	// Set "magic" user_process name, only if user did not specified one.
	if ( magic_user_process_set == 0) {
		strncpy(img1.magic_user_process,
			os_to_set->magic_user_process_name,
			TS_COMM_LEN);
	}

	// Set "magic" process name, only if user did not specified one.
	if ( magic_module_set == 0) {
		strncpy(img1.magic_module,
			os_to_set->magic_module_name,
			MD_NAME_LEN);
	}

	// Set pointer to OS structure itself:
	img1.os1=os_to_set;

	return 0;
}

/*
 * Function that sets pointer to correct functions,
 * according to detected os
 * This must be done by checking string;(
 * Index in array can change.
 *
 * set_os_from_list (string with os name)
 * return 0 on success, otherwise error > 0
 * results are stored in global structure
 *
 */


int set_os_from_list(char *os_name){
	int status,j;

	dbgprint("OS: %s",os_name);

	for (status=0,j=1;j<=img1.oscount;j++) {
		// Compare user specified os and one of list		
		status=strncmp(systems[j].operating_system,
			os_name,
			OS_NAME_LEN);

		if (status == 0) {
			dbgprint("Setting os: %s (%d)",
				os_name,j);
			os_settings(&systems[j]);
			img1.os=j; //set index
			return 0;
		}
	}
	return 1;
}

